"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.default = exports.temporaryErrorHttpCodes = exports.HttpError = exports.FetchError = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));

var _wrapNativeSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/wrapNativeSuper"));

var _get_root_based_url = _interopRequireDefault(require("./get_root_based_url"));

var _sync_jwt_provider_with_store = _interopRequireDefault(require("./sync_jwt_provider_with_store"));

var FetchError =
/*#__PURE__*/
function (_Error) {
  (0, _inheritsLoose2.default)(FetchError, _Error);

  function FetchError() {
    return _Error.apply(this, arguments) || this;
  }

  return FetchError;
}((0, _wrapNativeSuper2.default)(Error));

exports.FetchError = FetchError;

var HttpError =
/*#__PURE__*/
function (_Error2) {
  (0, _inheritsLoose2.default)(HttpError, _Error2);

  function HttpError() {
    return _Error2.apply(this, arguments) || this;
  }

  return HttpError;
}((0, _wrapNativeSuper2.default)(Error));

exports.HttpError = HttpError;
var temporaryErrorHttpCodes = [408, // Request Timeout
429, // Too Many Requests
502, // Bad Gateway
503, // Service Unavailable
504, // Gateway Timeout
507, // Insufficient Storage
509, // Bandwidth Limit Exceeded
521, // Web Server Is Down
522, // Connection Timed Out
523, // Origin Is Unreachable
524 // A Timeout Occurred
];
exports.temporaryErrorHttpCodes = temporaryErrorHttpCodes;

var validateStatus = function validateStatus(status) {
  // eslint-disable-next-line eqeqeq
  if (temporaryErrorHttpCodes.find(function (code) {
    return code == status;
  })) {
    throw new FetchError(status);
  }
};

var createApi = function createApi(_ref) {
  var origin = _ref.origin,
      rootPath = _ref.rootPath,
      jwtProvider = _ref.jwtProvider,
      store = _ref.store;

  var _request =
  /*#__PURE__*/
  function () {
    var _ref2 = (0, _asyncToGenerator2.default)(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee(url, body) {
      var rootBasedUrl, options, jwtToken, response, responseJwtToken;
      return _regenerator.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              rootBasedUrl = (0, _get_root_based_url.default)(origin, rootPath, url);
              options = {
                method: 'POST',
                headers: {
                  'Content-Type': 'application/json'
                },
                credentials: 'same-origin',
                body: JSON.stringify(body)
              };

              if (!jwtProvider) {
                _context.next = 7;
                break;
              }

              _context.next = 5;
              return jwtProvider.get();

            case 5:
              jwtToken = _context.sent;

              if (jwtToken) {
                options.headers.Authorization = "Bearer " + jwtToken;
              }

            case 7:
              _context.next = 9;
              return fetch(rootBasedUrl, options);

            case 9:
              response = _context.sent;
              responseJwtToken = (response.headers.get('Authorization') || response.headers.get('authorization') || '').replace(/^Bearer /i, '');

              if (!jwtProvider) {
                _context.next = 14;
                break;
              }

              _context.next = 14;
              return jwtProvider.set(responseJwtToken);

            case 14:
              _context.next = 16;
              return (0, _sync_jwt_provider_with_store.default)(jwtProvider, store);

            case 16:
              return _context.abrupt("return", response);

            case 17:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function request(_x, _x2) {
      return _ref2.apply(this, arguments);
    };
  }();

  return {
    loadViewModelState: function () {
      var _loadViewModelState = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee2(_ref3) {
        var viewModelName, aggregateIds, aggregateArgs, response, result, queryAggregateIds;
        return _regenerator.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                viewModelName = _ref3.viewModelName, aggregateIds = _ref3.aggregateIds, aggregateArgs = _ref3.aggregateArgs;
                _context2.prev = 1;
                queryAggregateIds = aggregateIds === '*' ? aggregateIds : aggregateIds.join(',');
                _context2.next = 5;
                return _request("/api/query/" + viewModelName + "/" + queryAggregateIds, {
                  aggregateArgs: aggregateArgs
                });

              case 5:
                response = _context2.sent;
                _context2.next = 11;
                break;

              case 8:
                _context2.prev = 8;
                _context2.t0 = _context2["catch"](1);
                throw new FetchError(_context2.t0.message);

              case 11:
                validateStatus(response.status);

                if (response.ok) {
                  _context2.next = 18;
                  break;
                }

                _context2.t1 = HttpError;
                _context2.next = 16;
                return response.text();

              case 16:
                _context2.t2 = _context2.sent;
                throw new _context2.t1(_context2.t2);

              case 18:
                _context2.prev = 18;
                _context2.next = 21;
                return response.text();

              case 21:
                result = _context2.sent;
                _context2.next = 27;
                break;

              case 24:
                _context2.prev = 24;
                _context2.t3 = _context2["catch"](18);
                throw new HttpError(_context2.t3.message);

              case 27:
                return _context2.abrupt("return", {
                  timestamp: Number(new Date(response.headers.get('Date'))),
                  result: result
                });

              case 28:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this, [[1, 8], [18, 24]]);
      }));

      return function loadViewModelState(_x3) {
        return _loadViewModelState.apply(this, arguments);
      };
    }(),
    loadReadModelState: function () {
      var _loadReadModelState = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee3(_ref4) {
        var readModelName, resolverName, resolverArgs, response, result;
        return _regenerator.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                readModelName = _ref4.readModelName, resolverName = _ref4.resolverName, resolverArgs = _ref4.resolverArgs;
                _context3.prev = 1;
                _context3.next = 4;
                return _request("/api/query/" + readModelName + "/" + resolverName, resolverArgs);

              case 4:
                response = _context3.sent;
                _context3.next = 10;
                break;

              case 7:
                _context3.prev = 7;
                _context3.t0 = _context3["catch"](1);
                throw new FetchError(_context3.t0.message);

              case 10:
                validateStatus(response.status);

                if (response.ok) {
                  _context3.next = 17;
                  break;
                }

                _context3.t1 = HttpError;
                _context3.next = 15;
                return response.text();

              case 15:
                _context3.t2 = _context3.sent;
                throw new _context3.t1(_context3.t2);

              case 17:
                _context3.prev = 17;
                _context3.next = 20;
                return response.text();

              case 20:
                result = _context3.sent;
                _context3.next = 26;
                break;

              case 23:
                _context3.prev = 23;
                _context3.t3 = _context3["catch"](17);
                throw new HttpError(_context3.t3.message);

              case 26:
                return _context3.abrupt("return", {
                  timestamp: Number(new Date(response.headers.get('Date'))),
                  result: result
                });

              case 27:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this, [[1, 7], [17, 23]]);
      }));

      return function loadReadModelState(_x4) {
        return _loadReadModelState.apply(this, arguments);
      };
    }(),
    sendCommand: function () {
      var _sendCommand = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee4(_ref5) {
        var commandType, aggregateId, aggregateName, payload, response;
        return _regenerator.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                commandType = _ref5.commandType, aggregateId = _ref5.aggregateId, aggregateName = _ref5.aggregateName, payload = _ref5.payload;
                _context4.prev = 1;
                _context4.next = 4;
                return _request('/api/commands', {
                  type: commandType,
                  aggregateId: aggregateId,
                  aggregateName: aggregateName,
                  payload: payload
                });

              case 4:
                response = _context4.sent;
                _context4.next = 10;
                break;

              case 7:
                _context4.prev = 7;
                _context4.t0 = _context4["catch"](1);
                throw new FetchError(_context4.t0.message);

              case 10:
                validateStatus(response.status);

                if (response.ok) {
                  _context4.next = 17;
                  break;
                }

                _context4.t1 = HttpError;
                _context4.next = 15;
                return response.text();

              case 15:
                _context4.t2 = _context4.sent;
                throw new _context4.t1(_context4.t2);

              case 17:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this, [[1, 7]]);
      }));

      return function sendCommand(_x5) {
        return _sendCommand.apply(this, arguments);
      };
    }(),
    getSubscribeAdapterOptions: function () {
      var _getSubscribeAdapterOptions = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee5(adapterName) {
        var response, result;
        return _regenerator.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.prev = 0;
                _context5.next = 3;
                return _request('/api/subscribe', {
                  origin: origin,
                  rootPath: rootPath,
                  adapterName: adapterName
                });

              case 3:
                response = _context5.sent;
                _context5.next = 9;
                break;

              case 6:
                _context5.prev = 6;
                _context5.t0 = _context5["catch"](0);
                throw new FetchError(_context5.t0.message);

              case 9:
                validateStatus(response.status);

                if (response.ok) {
                  _context5.next = 16;
                  break;
                }

                _context5.t1 = HttpError;
                _context5.next = 14;
                return response.text();

              case 14:
                _context5.t2 = _context5.sent;
                throw new _context5.t1(_context5.t2);

              case 16:
                _context5.prev = 16;
                _context5.next = 19;
                return response.json();

              case 19:
                result = _context5.sent;
                _context5.next = 25;
                break;

              case 22:
                _context5.prev = 22;
                _context5.t3 = _context5["catch"](16);
                throw new HttpError(_context5.t3.message);

              case 25:
                return _context5.abrupt("return", result);

              case 26:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this, [[0, 6], [16, 22]]);
      }));

      return function getSubscribeAdapterOptions(_x6) {
        return _getSubscribeAdapterOptions.apply(this, arguments);
      };
    }(),
    request: function () {
      var _request2 = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee6(_ref6) {
        var url, body;
        return _regenerator.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                url = _ref6.url, body = _ref6.body;
                return _context6.abrupt("return", _request(url, body));

              case 2:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      return function request(_x7) {
        return _request2.apply(this, arguments);
      };
    }()
  };
};

var _default = createApi;
exports.default = _default;
//# sourceMappingURL=create_api.js.map