"use strict";

exports.__esModule = true;
exports.default = void 0;

var getAddedConnections = function getAddedConnections(prevConnections, nextConnections) {
  return nextConnections.filter(function (connection) {
    return !prevConnections.find(function (_ref) {
      var connectionName = _ref.connectionName,
          connectionId = _ref.connectionId;
      return connection.connectionName === connectionName && connection.connectionId === connectionId;
    });
  });
};

var getRemovedConnections = function getRemovedConnections(prevConnections, nextConnections) {
  return prevConnections.filter(function (connection) {
    return !nextConnections.find(function (_ref2) {
      var connectionName = _ref2.connectionName,
          connectionId = _ref2.connectionId;
      return connection.connectionName === connectionName && connection.connectionId === connectionId;
    });
  });
};

var createConnectionManager = function createConnectionManager(_temp) {
  var _ref3 = _temp === void 0 ? {} : _temp,
      _ref3$wildcardSymbol = _ref3.wildcardSymbol,
      wildcardSymbol = _ref3$wildcardSymbol === void 0 ? '*' : _ref3$wildcardSymbol;

  var connections = Object.create(null);

  var getConnections = function getConnections() {
    var result = [];

    for (var connectionName in connections) {
      var connectionsByName = [];

      for (var connectionId in connections[connectionName]) {
        if (connectionId === wildcardSymbol) {
          connectionsByName = [{
            connectionName: connectionName,
            connectionId: connectionId
          }];
          break;
        }

        connectionsByName.push({
          connectionName: connectionName,
          connectionId: connectionId
        });
      }

      result.push.apply(result, connectionsByName);
    }

    return result;
  };

  return {
    addConnection: function addConnection(_ref4) {
      var connectionName = _ref4.connectionName,
          connectionId = _ref4.connectionId;
      var prevConnections = getConnections();

      if (!connections[connectionName]) {
        connections[connectionName] = Object.create(null);
      }

      if (!connections[connectionName][connectionId]) {
        connections[connectionName][connectionId] = 0;
      }

      connections[connectionName][connectionId]++;
      var nextConnections = getConnections();
      var addedConnections = getAddedConnections(prevConnections, nextConnections);
      var removedConnections = getRemovedConnections(prevConnections, nextConnections);
      return {
        addedConnections: addedConnections,
        removedConnections: removedConnections
      };
    },
    removeConnection: function removeConnection(_ref5) {
      var connectionName = _ref5.connectionName,
          connectionId = _ref5.connectionId;
      var prevConnections = getConnections();
      connections[connectionName][connectionId]--;

      if (!connections[connectionName][connectionId]) {
        delete connections[connectionName][connectionId];
      }

      if (Object.keys(connections[connectionName]).length === 0) {
        delete connections[connectionName];
      }

      var nextConnections = getConnections();
      var addedConnections = getAddedConnections(prevConnections, nextConnections);
      var removedConnections = getRemovedConnections(prevConnections, nextConnections);
      return {
        addedConnections: addedConnections,
        removedConnections: removedConnections
      };
    }
  };
};

var _default = createConnectionManager;
exports.default = _default;
//# sourceMappingURL=create_connection_manager.js.map